# -*- coding: utf-8 -*-

from fabric.api import *
from fabez.cmd import cmd_expanduser,cmd_su
import os


def put_public_key(path=None, user=None):
    """
    Upload pub key from remote server.
    Limit: Openssh standard key,must comment with user mail.
    :param path:local path
    :param user:remote username
    :return:
    """
    if os.path.exists(os.path.expanduser(path)) is False:
        abort("public key not exist")
    else:
        # 通过解读最后注释来判断key是否存在，如果不存在注释，判断为非法的key
        fp = open(os.path.expanduser(path))
        pub_key = fp.read()
        pos = pub_key.rfind(" ")

        mail = pub_key[pos + 1:].strip()
        if mail.find('@') == -1:
            abort('please add comment WHO YOU ARE.')

    if user:
        user_path = cmd_expanduser(user)
    else:
        user_path = '~'

    remote_root = '%s/.ssh' % user_path
    remote_path = '%s/authorized_keys' % remote_root

    with settings(warn_only=True):
        if run('test -d %s' % remote_root).failed:
            cmd_su('mkdir %s' % remote_root, user)
            if user:
                run('chown %s.%s %s' % (user, user, remote_root))

    put(path, '/tmp/tmp.pub', mode=0644)
    cmd_su('grep %s %s | cat /tmp/tmp.pub >> %s' % (mail, remote_path, remote_path),user)
    if user:
        run('chown %s.%s %s' % (user, user, remote_path))

    pass


def put_private_key(path=None, user=None):
    """
    Upload private key to remote server
    Limit: Must be a standard key generate from ssh-keygen
    :param path:local path
    :param user:remote username
    """
    if os.path.exists(os.path.expanduser(path)) is False:
        abort("private key not exist")
    else:
        # valid key type
        fp = open(os.path.expanduser(path))
        private_key = fp.read()
        pos = private_key.find("\n")

        if private_key[0:pos].find('DSA') > -1:
            dsa = True
        else:
            dsa = False

    user_path = cmd_expanduser(user)

    remote_root = '%s/.ssh' % user_path

    if dsa:
        remote_path = '%s/id_dsa' % remote_root
    else:
        remote_path = '%s/id_rsa' % remote_root

    with settings(warn_only=True):
        if run('test -d %s' % remote_root).failed:
            if user:
                run('chown -Rf %s.%s %s' % (user, user, user_path))
            cmd_su('mkdir %s' % remote_root,user)

    put(path, remote_path, mode=0600)
    if user:
        run('chown %s.%s %s' % (user, user, remote_path))
    pass




# -*- coding: utf-8 -*-

from fabric.api import *


def utils_git(proxy=None,ignore_perm=True):
    '''
    Install git
    :param proxy:proxy server
    :return:
    '''
    run('yum install git -y')

    if proxy:
        run('git config --global http.proxy {}'.format(proxy))

    if ignore_perm:
        run('git config --global core.fileMode false')

    pass



def rm_utils_git():
    '''
    remove git and git config
    :return:
    '''
    with settings(warn_only=True):
        run('yum erase git -y')
        run('rm -rf ~/.gitconfig')





def utils_baselib():
    '''
    install base package
    :return:
    '''
    run(
        'yum install gcc-c++ make zlib-devel libxml2  libxml2-devel  bzip2-libs bzip2-devel  libcurl-devel  libjpeg libjpeg-devel  libpng libpng-devel  freetype freetype-devel  libmcrypt libmcrypt-devel  libtool-ltdl libtool-ltdl-devel openssl-devel autoconf re2c bison pcre_devel pcre   libtiff-devel libjpeg-devel libzip-devel freetype-devel lcms2-devel libwebp-devel tcl-devel tk-devel -y')
    run('mkdir -p /usr/local/var/run')
    pass





def utils_epel():
    """
    install epel lib
    :return:
    """
    with settings(warn_only=True):
        run('rpm -Uvh http://dl.fedoraproject.org/pub/epel/6/x86_64/epel-release-6-8.noarch.rpm')
    pass



def utils_remi():
    """
    install epel lib
    :return:
    """
    with settings(warn_only=True):
        run('rpm -Uvh http://rpms.famillecollet.com/enterprise/remi-release-6.rpm')
    pass




def utils_hg(proxy=None):
    '''
    Install Mercurial
    :param proxy:proxy server
    :return:
    '''
    run('yum install mercurial -y')

    pass






# -*- coding: utf-8 -*-

from fabric.api import *

def io_tmpfs(path,boot=True,size=8,uid='root',gid='root',mode='777'):
    """
    tmpfs function.
    @todo uid,gid feature
    :param path:
    :param boot:
    :param size:
    :param uid:
    :param gid:
    :param mode:
    :return:
    """

    match = "tmpfs    %s" % path
    replace = "%s    tmpfs    uid=root,gid=root,size=%sG,mode=777  0 0" % (match, size)

    if boot is True:
        with settings(warn_only=True):
            run('mkdir %s' % path)
            run('grep "%s" /etc/fstab && sed -i -e "s/%s.*//g" /etc/fstab' % (match, '\/'.join(match.split('/'))))
        run('grep "%s" /etc/fstab || echo "%s" >> /etc/fstab ' % (match, replace))

        with settings(warn_only=True):
            if run('df -h | grep %s' % path).failed:
                run('mount %s' % path)

    pass



def rm_io_tmpfs(path,boot=True):
    with settings(warn_only=True):
        if run('df -h | grep %s' % path):
            run('umount %s' % path)


    if boot is True:
        match = "tmpfs    %s" % path
        with settings(warn_only=True):
            run('grep "%s" /etc/fstab && sed -i -e "s/%s.*//g" /etc/fstab' % (match, '\/'.join(match.split('/'))))
    pass


def io_disk(real_path,uid=None,gid=None,mode='777'):
    """
    base function
    :param real_path:
    :param uid:
    :param gid:
    :param mode:
    :return:
    """
    with settings(warn_only=True):
        run('mkdir -p {}'.format(real_path))

    if gid is None:
        gid = uid

    if uid is None:
        run('chmod {} -Rf {}'.format(mode,real_path))
    else:
        run('chown -Rf {}.{} {} '.format(uid,gid,real_path))
        run('chmod 700 -Rf {}'.format(real_path))

    pass


def rm_io_disk(path):
    return run('rm -rf %s' % path)



def io_airlog():
    """
    FastLog
    :return:
    """
    return io_tmpfs('/airlog')



def rm_io_airlog():
    return rm_io_tmpfs('/airlog')



def io_webdata(uid='webuser',gid='webuser',tmpfs=False):
    if tmpfs:
        return io_tmpfs('/webdata',uid=uid,gid=gid,mode=700)
    else:
        return io_disk('/webdata',uid=uid,gid=gid)



def rm_io_webdata(uid='webuser',gid='webuser',tmpfs=False):
    if tmpfs:
        return rm_io_tmpfs('/webdata',uid=uid,gid=gid)
    else:
        return rm_io_disk('/webdata',uid=uid,gid=gid)



def io_slowlog(path,user=None):
    """
    create slow log system
    :return:
    """

    real_path = '/logs/%s' % path
    return io_disk(real_path,uid=user)
    pass


def rm_io_slowlog(path):
    return rm_io_disk(path)



def io_socks(path,user=None):
    """
    create slow log system
    :return:
    """

    real_path = '/usr/local/var/run/%s' % path
    io_disk(real_path,uid=user)
    pass


def rm_io_socks(path):
    return rm_io_disk(path)




# 问题来自于部署oc的机器时,发现内存耗尽了.
# https://www.digitalocean.com/community/tutorials/how-to-add-swap-on-centos-6
# 5个G
# dd if=/dev/zero of=/swapfile bs=10240 count=512k
# mkswap /swapfile
# /etc/fstab
# /swapfile          swap            swap    defaults        0 0

#chown root:root /swapfile
#chmod 0600 /swapfile


